@@ -58,41 +58,38 @@ x <- y <- value <- NULL
 #' # print(chart)
 waffle <- function(parts, rows=10, xlab=NULL, title=NULL, colors=NA,
                    size=2, flip=FALSE, reverse=FALSE, equal=TRUE, pad=0,
-                   use_glyph=FALSE, glyph_size=12,legend_pos="right") {
+                   use_glyph=FALSE, glyph_size=12, legend_pos="right") {
 
   # fill in any missing names
-
   part_names <- names(parts)
   if (length(part_names) < length(parts)) {
     part_names <- c(part_names, LETTERS[1:length(parts)-length(part_names)])
   }
 
-  # use Set2 if no colors are specified
+  names(parts) <- part_names
 
-  if (all(is.na(colors))) {
-    colors <- brewer.pal(length(parts), "Set2")
-  }
+  # use Set2 if no colors are specified
+  if (all(is.na(colors))) colors <- suppressWarnings(brewer.pal(length(parts), "Set2"))
 
   # make one big vector of all the bits
-
   parts_vec <- unlist(sapply(1:length(parts), function(i) {
-    rep(LETTERS[i+1], parts[i])
+    rep(names(parts)[i], parts[i])
   }))
 
-  if (reverse) { parts_vec <- rev(parts_vec) }
+  if (reverse) parts_vec <- rev(parts_vec)
 
   # setup the data frame for geom_rect
-
   dat <- expand.grid(y=1:rows, x=seq_len(pad + (ceiling(sum(parts) / rows))))
 
   # add NAs if needed to fill in the "rectangle"
-
   dat$value <- c(parts_vec, rep(NA, nrow(dat)-length(parts_vec)))
   if(!inherits(use_glyph, "logical")){
       fontlab <- rep(fa_unicode[use_glyph],length(unique(parts_vec)))
       dat$fontlab <- c(fontlab[as.numeric(factor(parts_vec))], rep(NA, nrow(dat)-length(parts_vec)))
   }
 
+  dat$value <- factor(dat$value, levels=part_names)
+
   if (flip) {
     gg <- ggplot(dat, aes(x=y, y=x))
   } else {
@@ -108,8 +105,11 @@ waffle <- function(parts, rows=10, xlab=NULL, title=NULL, colors=NA,
     gg <- gg + geom_tile(aes(fill=value), color="white", size=size)
     gg <- gg + scale_fill_manual(name="",
                                  values=colors,
-                                 labels=part_names)
-    gg <- gg + guides(fill=guide_legend(override.aes=list(colour=NULL)))
+                                 labels=part_names,
+                                 drop=FALSE)
+    gg <- gg + guides(fill=guide_legend(override.aes=list(colour="#00000000")))
+    gg <- gg + theme(legend.background=element_rect(fill="#00000000", color="#00000000"))
+    gg <- gg + theme(legend.key=element_rect(fill="#00000000", color="#00000000"))
 
   } else {
 
@@ -128,16 +128,17 @@ waffle <- function(parts, rows=10, xlab=NULL, title=NULL, colors=NA,
       message("Font Awesome by Dave Gandy - http://fontawesome.io")
     }
 
-    gg <- gg + geom_tile(color=NA, fill=NA, size=size, alpha=0, show_guide=FALSE)
-    gg <- gg + geom_point(aes(color=value), fill=NA, size=0, show_guide=TRUE)
+    gg <- gg + geom_tile(color=NA, fill=NA, size=size, alpha=0, show.legend=FALSE)
+    gg <- gg + geom_point(aes(color=value), fill=NA, size=0, show.legend=TRUE)
     gg <- gg + geom_text(aes(color=value,label=fontlab),
-                         family="FontAwesome", size=glyph_size, show_guide=FALSE)
+                         family="FontAwesome", size=glyph_size, show.legend=FALSE)
     gg <- gg + scale_color_manual(name="",
                                  values=colors,
-                                 labels=part_names)
+                                 labels=part_names,
+                                 drop=FALSE)
     gg <- gg + guides(color=guide_legend(override.aes=list(shape=15, size=7)))
-    gg <- gg + theme(legend.background=element_rect(fill=NA, color=NA))
-    gg <- gg + theme(legend.key=element_rect(color=NA))
+    gg <- gg + theme(legend.background=element_rect(fill="#00000000", color="#00000000"))
+    gg <- gg + theme(legend.key=element_rect(color="#00000000"))
 
   }
 
